/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Event Logger.
 *
 * The Initial Developer of the Original Code is
 *   Ben Basson <contact@cusser.net>
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Basson <contact@cusser.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var eventloggerWindow = 
{
  appPrefs: null,
  listBox: null,
  lastFile: null,
  changes: false,
  comparison: false,
  
  init: function ()
  {
    var prefs = Components.classes['@mozilla.org/preferences-service;1']
                  .getService(Components.interfaces.nsIPrefService);
    var appPrefs = eventloggerWindow.appPrefs = prefs.getBranch('eventlogger.');
    
    eventloggerUpdateObserver.register();

    eventloggerWindow.setButtons(appPrefs.getBoolPref("logging"));
    window.removeEventListener("load", eventloggerWindow.init, false);
    window.addEventListener("unload", eventloggerWindow.uninit, false);

    var listBox = eventloggerWindow.listBox = document.getElementById("event-list");
    listBox.addEventListener("select",eventloggerWindow.setDeleteItem,false);
    listBox.addEventListener("keypress",eventloggerWindow.handleKeypress,false);
    eventloggerWindow.populateListBox(listBox, null);
    eventloggerWindow.setResetButton();
  },

  uninit: function ()
  {
    eventloggerUpdateObserver.unregister();  
  },
  
  startLogging: function ()
  {
    eventloggerWindow.appPrefs.setBoolPref("logging",true);
    eventloggerWindow.setButtons(true);
  },
  
  stopLogging: function ()
  {
    eventloggerWindow.appPrefs.setBoolPref("logging",false);
    eventloggerWindow.setButtons(false);
  },
  
  resetLog: function ()
  {
    if (eventloggerWindow.changes)
    {
      var button = eventloggerWindow.confirmPrompt();

      if (button == 0)
        eventloggerWindow.saveEventList(true);
      else if (button == 1)
        return false;   
    }
      
    var file = eventloggerUtils.getTempFile();
    if (file.exists())
      file.remove(false);
    eventloggerWindow.resetListBox();
    eventloggerWindow.setResetButton();
  },
  
  setButtons: function (aBool)
  {
    document.getElementById("startLogging").setAttribute("disabled",aBool);
    document.getElementById("stopLogging").setAttribute("disabled",!aBool);
    document.getElementById("openEventLog").setAttribute("disabled",aBool);
    document.getElementById("compareLogs").setAttribute("disabled",aBool);
    document.getElementById("saveEventLog").setAttribute("disabled",aBool);
    document.getElementById("saveEventLogAs").setAttribute("disabled",aBool);
    document.getElementById("exportLog").setAttribute("disabled",aBool);
    eventloggerWindow.setResetButton();
  },
  
  setDeleteItem: function ()
  {
    var listBox = eventloggerWindow.listBox;
    document.getElementById("deleteEvent").setAttribute("disabled", listBox.selectedCount == 0);
  },
  
  setResetButton: function ()
  {
    var listBox = eventloggerWindow.listBox;
    var logging = eventloggerWindow.appPrefs.getBoolPref("logging");
    var disable = true;
    if (listBox && listBox.childNodes.length > 0 && logging == false)
      disable = false;

    document.getElementById("copyEventLog").setAttribute("disabled", disable);
    document.getElementById("resetLog").setAttribute("disabled", disable);
    document.getElementById("compareLogs").setAttribute("disabled", disable);
  },
  
  populateListBox: function (aListBox, aFile)
  {
    var eventDoc = eventloggerUtils.getDOMTree(aFile);
    var fromFile = aFile ? true : false;
    if (eventDoc && eventDoc.documentElement && eventDoc.documentElement.nodeName == "eventlist")
    {
      var eventList = eventDoc.documentElement.childNodes;
      if (eventList.length && eventList.length > 0)
        for (var n = 0; n < eventList.length; n++)
          eventloggerWindow.addEvent(aListBox, eventList[n], fromFile);
    }
  },
  
  addEvent: function (aListBox, aEventNode, aFromFile)
  {
    if (!aEventNode)
      return;
    
    const xulxmlns = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    var listBox = aListBox ? aListBox : eventloggerWindow.listBox;

    var newEvent = document.createElementNS(xulxmlns, "listitem");
    var normalLabel = aEventNode.getAttribute("eventDescription");
    var changePointLabel =aEventNode.getAttribute("changeDescription");
    var changePoint = aEventNode.getAttribute("changePoint");
    
    if (changePoint)
      newEvent.setAttribute("label", changePointLabel);
    else
      newEvent.setAttribute("label", normalLabel);
    
    newEvent.setAttribute("normalLabel", normalLabel);
    newEvent.setAttribute("changePointLabel", changePointLabel);
    newEvent.setAttribute("changePoint", changePoint);
    newEvent.setAttribute("fromFile", aFromFile ? true : false);

    var rows = listBox.getRowCount();
    listBox.appendChild(newEvent);
    listBox.ensureIndexIsVisible(rows);
    
    if (!aFromFile)
      eventloggerWindow.changes = true;
  },
  
  removeEvent: function (aNode)
  {
    eventloggerWindow.listBox.removeChild(aNode);
  },

  resetListBox: function (aListBox)
  {
    var listBox = aListBox ? aListBox : eventloggerWindow.listBox;
    for (var n = listBox.childNodes.length - 1; n >= 0; n--)
      listBox.removeChild(listBox.childNodes[n]);
  },
  
  deleteEvent: function ()
  {
    var listBox = eventloggerWindow.listBox;
    var index = listBox.selectedIndex;
    if (index == -1)
      return;
    
    if (listBox.childNodes[index].getAttribute("changePoint"))
    {
      var nextNode = listBox.childNodes[index+1];
      if (nextNode && !nextNode.getAttribute("changePoint"))
      {
        nextNode.setAttribute("label", nextNode.getAttribute("changePointLabel"));
        nextNode.setAttribute("changePoint", true);
      }
    }
        
    // specify if the event came from a loaded file or not
    // and notify the main code
    var fromFile = listBox.childNodes[index].getAttribute("fromFile") == "true" ? true : false;
    eventloggerUtils.notifyObservers("deleteEvent", [index, fromFile])

    eventloggerWindow.removeEvent(listBox.selectedItem);
    
    var maxNode = listBox.childNodes.length - 1;
    index = (index > maxNode) ? maxNode : index;
    listBox.selectedIndex = index;
    eventloggerWindow.setDeleteItem();
    eventloggerWindow.setResetButton();
    eventloggerWindow.changes = true;
    if (eventloggerWindow.comparison == true)
    {
      eventloggerWindow.clearComparison();
      eventloggerWindow.compareLists();
    }
  },
  
  handleKeypress: function (aEvent)
  {
    if (aEvent.keyCode && aEvent.keyCode == 46)
      eventloggerWindow.deleteEvent();
  },
  
  openEventList: function (aListBox)
  {
    var listBox = aListBox ? aListBox : eventloggerWindow.listBox;
    
    // check before replacing document
    if (listBox.childNodes.length > 0 && eventloggerWindow.changes == true)
    {
      var button = eventloggerWindow.confirmPrompt();

      if (button == 0)
        eventloggerWindow.saveEventList(true);
      else if (button == 1)
        return false;
    }

    var file = eventloggerUtils.filePicker(false);

    // store for quick-saving
    if (file)
      eventloggerWindow.lastFile = file.path
    else
      return false;

    // synchronises temp file with loaded file
    var eventDoc = eventloggerUtils.getDOMTree(file);
    
    eventloggerUtils.outputEventList(eventDoc,true)
    eventloggerWindow.resetListBox(listBox);
    eventloggerWindow.populateListBox(listBox, file);
    eventloggerWindow.setResetButton();
    eventloggerUtils.notifyObservers("loadFile")
    eventloggerWindow.changes = false;
  },

  saveEventList: function (aDirectSave)
  {
    var outputFile = null;
    if (aDirectSave && eventloggerWindow.lastFile)
    {
      outputFile = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
      outputFile.initWithPath(eventloggerWindow.lastFile);
    }
    
    var file = eventloggerUtils.getTempFile();
    var eventDoc = eventloggerUtils.getDOMTree(file);
    eventloggerWindow.lastFile = eventloggerUtils.outputEventList(eventDoc, outputFile ? outputFile : false);
    eventloggerWindow.changes = false;
  },
  
  confirmPrompt: function ()
  {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                         .getService(Components.interfaces.nsIPromptService);         
    /* 
      XXX this is the worst XPCOM function in history
      See http://ihoss.not-a-blog.com/prompt.php
      Note: button order is 0, 2, 1.
    */
    var stringBundle = document.getElementById("eventloggerStrings");
    var title = stringBundle.getString("eventlogger");
    var confirmtext = stringBundle.getString("confirmtext");
    return promptService.confirmEx(window, title, confirmtext, 1*5+65536*6+256*2, null, null, null, null, {});
  },
  
  exportText: function (aToClipboard)
  {
    var outputText = new String();
    var listBox = eventloggerWindow.listBox;
    var counter = 1;
    var nodes = listBox.childNodes.length;
    
    for (var n = 0; n < nodes; n++)
      outputText += (n+1) + ". " + listBox.childNodes[n].label + "\n";
    
    if (aToClipboard)
    {
      var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                              .getService(Components.interfaces.nsIClipboardHelper);
      clipboardHelper.copyString(outputText);
    }
    else
    {
      var outputStream = Components.classes['@mozilla.org/network/file-output-stream;1']
                           .createInstance(Components.interfaces.nsIFileOutputStream);
      var file = eventloggerUtils.filePicker(true,true);
      
      outputStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
      outputStream.write(outputText, outputText.length);
      outputStream.close();
    }
  },
  
  toggleComparisonPane: function ()
  {
    var paneContainer = document.getElementById("event-pane-container");
    var listBox2 = document.getElementById("event-list-2");
    if (listBox2)
    {
      paneContainer.removeChild(listBox2);
      document.defaultView.innerWidth = document.defaultView.innerWidth / 2;
      document.getElementById("hiddenButtons").style.display="none";
      document.getElementById("compareLogs").focus();
      eventloggerWindow.comparison = false;
      eventloggerWindow.clearComparison();
      document.getElementById("startLogging").setAttribute("disabled",false);
      document.getElementById("compareLogs").setAttribute("disabled",false);
    }
    else
    {    
      var listBox = eventloggerWindow.listBox;
      var newListBox = listBox.cloneNode(false);
      newListBox.setAttribute("id","event-list-2");
      paneContainer.appendChild(newListBox);
      document.defaultView.innerWidth = 2 * document.defaultView.innerWidth;
      document.getElementById("hiddenButtons").style.display="block";
      eventloggerWindow.comparison = true;
      eventloggerWindow.compareLists();
      document.getElementById("startLogging").setAttribute("disabled",true);
      document.getElementById("compareLogs").setAttribute("disabled",true);
    }
  },
  
  loadComparison: function ()
  {
    var listBox2 = document.getElementById("event-list-2");
    eventloggerWindow.openEventList(listBox2);
    window.setTimeout("eventloggerWindow.clearComparison()",5);
    window.setTimeout("eventloggerWindow.compareLists()",10);
    document.getElementById("compareLogs").setAttribute("disabled",true);
  },
  
  compareLists: function ()
  {
    eventloggerWindow.clearComparison();
    var listBox = eventloggerWindow.listBox;
    var listBox2 = document.getElementById("event-list-2");
    var lastMatch = null;
    
    for (var n = listBox2.childNodes.length-1; n >= 0; n--)
    {
      for (var i = listBox.childNodes.length-1; i >=0; i--)
      if (listBox2.childNodes[n].label == listBox.childNodes[i].label)
      {
        listBox2.childNodes[n].style.backgroundColor = "orange";
        listBox.childNodes[i].style.backgroundColor = "orange";
      }
    }
  },
  
  clearComparison: function ()
  {
    var listBox = eventloggerWindow.listBox;
    var listBox2 = document.getElementById("event-list-2");
    if (listBox2 && listBox2.childNodes.length > 0)
      for (var n = 0; n < listBox2.childNodes.length; n++)
        listBox2.childNodes[n].style.backgroundColor = "";
    if (listBox && listBox.childNodes.length > 0)
      for (var i = 0; i < listBox.childNodes.length; i++)
        listBox.childNodes[i].style.backgroundColor = "";
  }
};

var eventloggerUpdateObserver =
{
  register: function()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.addObserver(this, "appendEvent", false);
    os.addObserver(this, "removeEvent", false);
  },

  unregister: function ()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.removeObserver(this, "appendEvent", false);
    os.removeObserver(this, "removeEvent", false);
  },
  
  observe: function (aSubject, aTopic, aData)
  {
    if (aTopic == "appendEvent")
    {
      var parser = new DOMParser();
      var eventNode = parser.parseFromString(aData, "text/xml").firstChild;
      eventloggerWindow.addEvent(null, eventNode, false);
    }
    else
    {
      var listBox = eventloggerWindow.listBox;
      eventloggerWindow.removeEvent(listBox.lastChild);
    }
  }
};

window.addEventListener("load", eventloggerWindow.init, false);