/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Event Logger.
 *
 * The Initial Developer of the Original Code is
 *   Ben Basson <contact@cusser.net>
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Basson <contact@cusser.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var eventloggerUtils = 
{
  getTempFile: function ()
  {
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
      .getService(Components.interfaces.nsIProperties)
      .get("ProfD", Components.interfaces.nsIFile);
    file.append("extensions");
    file.append("{2AE23E44-ECD8-4D3B-864A-0B97F0726094}");
    file.append("eventDump.xml");
    return file;
  },
  
  getDOMTree: function (aFile)
  {
    var file = aFile ? aFile : eventloggerUtils.getTempFile();
    var uri = "file:///" + file.path;
    try
    {
      var req = new XMLHttpRequest();
      req.open("GET", uri, false);
      req.send(null);
    }
    catch (ex)
    {
      // don't throw an exception if file not found
    }
    
    if (req.responseXML)
      return req.responseXML;
    else
      return false;
  },
  
  outputEventList: function (aDoc, aFile)
  {
    var serializer = new XMLSerializer();
    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
      .createInstance(Components.interfaces.nsIFileOutputStream);
    var file;
    
    // if aFile is a file, use it, otherwise get temp file or filepicker
    if (aFile instanceof Components.interfaces.nsILocalFile)
      file = aFile;
    else
      file = aFile ? eventloggerUtils.getTempFile() : eventloggerUtils.filePicker(true);
    if (!file)
      return false;
    
    foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0);   // write, create, truncate
    serializer.serializeToStream(aDoc, foStream, "");
    foStream.close();
    return file.path;
  },
  
  filePicker: function (aSaving, aText)
  {
    const nsILocalFile = Components.interfaces.nsILocalFile;
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fpService = Components.classes["@mozilla.org/filepicker;1"]
    var filePicker = fpService.createInstance(nsIFilePicker);

    var mode = aSaving ? nsIFilePicker.modeSave : nsIFilePicker.modeOpen;
    var filter = aText ? nsIFilePicker.filterText : nsIFilePicker.filterXML;
    filePicker.defaultExtension = aText ? "txt" : "xml";
    filePicker.init(window, "", mode);
    filePicker.appendFilters(filter);
    
    if (filePicker.show() == nsIFilePicker.returnOK)
      return filePicker.file.QueryInterface(nsILocalFile);
    else
      return false;
  },
  
  notifyObservers: function (aType, aData)
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.notifyObservers(null, aType, aData);
  }
};